@@ -15,6 +15,7 @@ from uuid import uuid4
 
 from onnxconverter_common.registration import get_converter
 import onnx
+import timeit
 
 from hummingbird.ml._container import (
     PyTorchSklearnContainerRegression,
@@ -29,8 +30,13 @@ from hummingbird.ml._container import (
     ONNXSklearnContainerClassification,
     ONNXSklearnContainerTransformer,
     ONNXSklearnContainerAnomalyDetection,
+    TVMSklearnContainerRegression,
+    TVMSklearnContainerClassification,
+    TVMSklearnContainerTransformer,
+    TVMSklearnContainerAnomalyDetection,
+    BatchContainer,
 )
-from hummingbird.ml._utils import pandas_installed, _get_device
+from hummingbird.ml._utils import pandas_installed, tvm_installed, get_device, from_strings_to_ints
 from hummingbird.ml.exceptions import MissingConverter
 from hummingbird.ml.operator_converters import constants
 
@@ -40,29 +46,90 @@ else:
     DataFrame = None
 
 
-def _get_trace_input_from_test_input(input, batch_size):
+def _jit_model(torch_model, trace_input, device, extra_config):
+    """
+    Function used to convert an input pytorch model into torchscript.
+    """
+    if device != "cpu":
+        trace_input.to(device)
+    return torch.jit.trace(torch_model, trace_input).eval()
+
+
+def _get_trace_input_from_test_input(input, remainder_size=None, extra_config={}):
     """
     Utility function used to properly put the inputs into a format understandable by torch.
+    If `remainder_size` is provided, also return inputs for a remainder model (see below).
     """
-    if type(input) is tuple:
-        if batch_size is not None:
-            trace_input = tuple([torch.from_numpy(i)[0:batch_size, :] for i in input])
-        else:
-            trace_input = tuple([torch.from_numpy(i) for i in input])
+    remainder = None
+    if isinstance(input, tuple):
+        trace_input = []
+        for input_ in input:
+            # Convert string arrays into int32.
+            if input_.dtype.kind in constants.SUPPORTED_STRING_TYPES:
+                assert constants.MAX_STRING_LENGTH in extra_config
+                max_string_length = extra_config[constants.MAX_STRING_LENGTH]
+
+                input_ = from_strings_to_ints(input_, max_string_length)
+            trace_input.append(torch.from_numpy(input_))
+        trace_input = tuple(trace_input)
+        if remainder_size is not None and remainder_size != 0:
+            remainder = tuple([inp[0:remainder_size, :] for inp in trace_input])
     else:
+        # Convert string arrays into int32.
+        if input.dtype.kind in constants.SUPPORTED_STRING_TYPES:
+            assert constants.MAX_STRING_LENGTH in extra_config
+            max_string_length = extra_config[constants.MAX_STRING_LENGTH]
+
+            input = from_strings_to_ints(input, max_string_length)
         trace_input = torch.from_numpy(input)
-        if batch_size is not None:
-            trace_input = trace_input[0:batch_size, :]
-    return trace_input
+        if remainder_size is not None and remainder_size != 0:
+            remainder = trace_input[0:remainder_size, :]
+
+    return (trace_input, remainder)
 
 
-def convert(topology, backend, device, extra_config={}):
+def _get_batch_size(batch):
+    if isinstance(batch, tuple):
+        return batch[0].shape[0]
+
+    assert isinstance(batch, np.ndarray)
+    return batch.shape[0]
+
+
+def _compile_tvm_model(topology, torch_model, trace_input, target, ctx, config, extra_config):
+    import tvm
+    from tvm import relay
+    from tvm.contrib import graph_runtime
+
+    ts_model = _jit_model(torch_model, trace_input, "cpu", extra_config)
+    test_input = [
+        (topology.raw_model.input_names[i], trace_input[i].shape if type(trace_input) is tuple else trace_input.shape,)
+        for i in range(len(topology.raw_model.input_names))
+    ]
+
+    model, params = relay.frontend.from_pytorch(ts_model, test_input)
+
+    with tvm.transform.PassContext(opt_level=3, config=config):
+        graph, lib, params = relay.build(model, target=target, params=params)
+
+    tvm_model = graph_runtime.create(graph, lib, ctx)
+    tvm_model.set_input(**params)
+
+    extra_config[constants.TVM_GRAPH] = graph
+    extra_config[constants.TVM_LIB] = lib
+    extra_config[constants.TVM_PARAMS] = params
+
+    return tvm_model
+
+
+def convert(topology, backend, test_input, device, extra_config={}):
     """
     This function is used to convert a `onnxconverter_common.topology.Topology` object into a *backend* model.
 
     Args:
         topology: The `onnxconverter_common.topology.Topology` object that will be converted into a backend model
         backend: Which backend the model should be run on
+        test_input: Inputs for PyTorch model tracing
         device: Which device the translated model will be run on
         extra_config: Extra configurations to be used by individual operator converters
 
@@ -73,8 +140,14 @@ def convert(topology, backend, device, extra_config={}):
     assert backend is not None, "Cannot convert a Topology object into backend None."
     assert device is not None, "Cannot convert a Topology object into device None."
 
+    tvm_backend = None
     operator_map = {}
 
+    if tvm_installed():
+        import tvm
+
+        tvm_backend = tvm.__name__
+
     for operator in topology.topological_operator_iterator():
         try:
             converter = get_converter(operator.type)
@@ -99,7 +172,6 @@ def convert(topology, backend, device, extra_config={}):
 
     # Set the parameters for the model / container
     n_threads = None if constants.N_THREADS not in extra_config else extra_config[constants.N_THREADS]
-    batch_size = None if constants.BATCH_SIZE not in extra_config else extra_config[constants.BATCH_SIZE]
 
     # We set the number of threads for torch here to avoid errors in case we JIT.
     # We set intra op concurrency while we force operators to run sequentially.
@@ -114,6 +186,10 @@ def convert(topology, backend, device, extra_config={}):
         topology.raw_model.input_names, topology.raw_model.output_names, operator_map, operators, extra_config
     ).eval()
 
+    # if constants.REMAINDER_SIZE is present in extra_config, we are in the convert_batch mode.
+    remainder_model = None
+    remainder_size = None if constants.REMAINDER_SIZE not in extra_config else extra_config[constants.REMAINDER_SIZE]
+
     if backend == onnx.__name__:
         onnx_model_name = output_model_name = None
         target_opset = 11
@@ -128,12 +204,12 @@ def convert(topology, backend, device, extra_config={}):
             output_model_name = str(uuid4().hex) + ".onnx"
 
         # Put the tracing test input into the right format.
-        trace_input = _get_trace_input_from_test_input(extra_config[constants.TEST_INPUT], batch_size)
+        batch_trace_input, _ = _get_trace_input_from_test_input(test_input, remainder_size, extra_config)
 
         # Generate the ONNX models
         torch.onnx.export(
             torch_model,
-            trace_input,
+            batch_trace_input,
             output_model_name,
             input_names=topology.raw_model.input_names,
             output_names=topology.raw_model.output_names,
@@ -185,6 +261,43 @@ def convert(topology, backend, device, extra_config={}):
             return num_fixed
 
         fix_graph(hb_model.graph)
+    elif backend == tvm_backend:
+        # Pick the proper target.
+        if device == "cuda":
+            target = tvm.target.cuda()
+            ctx = tvm.gpu()
+        elif device == "cpu":
+            target = "llvm"
+            ctx = tvm.cpu()
+        elif "llvm" in device:
+            target = device
+            ctx = tvm.cpu()
+        else:
+            raise RuntimeError("Device {} not recognized".format(device))
+
+        # Get configuration parameters.
+        # 50 is a good depth for operator fusion. More than that will probably hurt performance.
+        # https://github.com/microsoft/hummingbird/issues/232#issuecomment-697979508
+        config = {"relay.FuseOps.max_depth": 50}
+
+        if constants.TVM_MAX_FUSE_DEPTH in extra_config:
+            config["relay.FuseOps.max_depth"] = extra_config[constants.TVM_MAX_FUSE_DEPTH]
+
+        # First we need to generate the torchscript model.
+        batch_trace_input, remainder_trace_input = _get_trace_input_from_test_input(test_input, remainder_size, extra_config)
+
+        tvm_model = _compile_tvm_model(topology, torch_model, batch_trace_input, target, ctx, config, extra_config)
+
+        if remainder_trace_input is not None:
+            remainder_model = _compile_tvm_model(
+                topology, torch_model, remainder_trace_input, target, ctx, config, extra_config
+            )
+
+        # In the container we will be using the context to properly configure the input tensors.
+        extra_config[constants.TVM_CONTEXT] = ctx
+        extra_config[constants.TVM_INPUT_NAMES] = topology.raw_model.input_names
+
+        hb_model = tvm_model
     else:
         # Set the device for the model.
         if device != "cpu":
@@ -193,18 +306,19 @@ def convert(topology, backend, device, extra_config={}):
 
         # If the backend is tochscript, jit the model.
         if backend == torch.jit.__name__:
-            trace_input = _get_trace_input_from_test_input(extra_config[constants.TEST_INPUT], batch_size)
+            trace_input, _ = _get_trace_input_from_test_input(test_input, remainder_size, extra_config)
             if device != "cpu":
                 trace_input.to(device)
             torch_model = torch.jit.trace(torch_model, trace_input).eval()
             torch.jit.optimized_execution(torch_model)
+
         hb_model = torch_model
 
     # Return if the container is not needed.
     if constants.CONTAINER in extra_config and not extra_config[constants.CONTAINER]:
         return hb_model
 
-    # We scan the operators backwards until we find an operator with a defined type
+    # We scan the operators backwards until we find an operator with a defined type.
     # This is necessary because ONNX models can have arbitrary operators doing casting, reshaping etc.
     idx = len(operators) - 1
     while (
@@ -232,12 +346,15 @@ def convert(topology, backend, device, extra_config={}):
         if idx < 0:
             idx = tmp_idx
 
+    # Get the proper container type.
     if operator_map[operators[idx].full_name].regression:
         # We are doing a regression task.
         if backend == torch.jit.__name__:
             container = TorchScriptSklearnContainerRegression
         elif backend == onnx.__name__:
             container = ONNXSklearnContainerRegression
+        elif backend == tvm_backend:
+            container = TVMSklearnContainerRegression
         else:
             container = PyTorchSklearnContainerRegression
     elif operator_map[operators[idx].full_name].anomaly_detection:
@@ -246,6 +363,8 @@ def convert(topology, backend, device, extra_config={}):
             container = TorchScriptSklearnContainerAnomalyDetection
         elif backend == onnx.__name__:
             container = ONNXSklearnContainerAnomalyDetection
+        elif backend == tvm_backend:
+            container = TVMSklearnContainerAnomalyDetection
         else:
             container = PyTorchSklearnContainerAnomalyDetection
     elif operator_map[operators[idx].full_name].transformer:
@@ -254,6 +373,8 @@ def convert(topology, backend, device, extra_config={}):
             container = TorchScriptSklearnContainerTransformer
         elif backend == onnx.__name__:
             container = ONNXSklearnContainerTransformer
+        elif backend == tvm_backend:
+            container = TVMSklearnContainerTransformer
         else:
             container = PyTorchSklearnContainerTransformer
     else:
@@ -262,14 +383,32 @@ def convert(topology, backend, device, extra_config={}):
             container = TorchScriptSklearnContainerClassification
         elif backend == onnx.__name__:
             container = ONNXSklearnContainerClassification
+        elif backend == tvm_backend:
+            container = TVMSklearnContainerClassification
         else:
             container = PyTorchSklearnContainerClassification
 
     n_threads = None if constants.N_THREADS not in extra_config else extra_config[constants.N_THREADS]
-    batch_size = None if constants.BATCH_SIZE not in extra_config else extra_config[constants.BATCH_SIZE]
-    hb_model = container(hb_model, n_threads, batch_size, extra_config=extra_config)
+    batch_size = None if constants.TEST_INPUT not in extra_config else _get_batch_size(test_input)
+    hb_container = container(hb_model, n_threads, batch_size, extra_config=extra_config)
 
-    return hb_model
+    if remainder_model:
+        aux_container = container(remainder_model, n_threads, remainder_size, extra_config=extra_config)
+        return BatchContainer(hb_container, aux_container)
+    elif remainder_size is not None and remainder_size > 0:
+        # remainder_size is non zero but remainder_model is not created
+        # -> torch backend case
+        aux_container = container(hb_model, n_threads, remainder_size, extra_config=extra_config)
+        return BatchContainer(hb_container, aux_container)
+    elif remainder_size is not None:
+        # remainder_size is not None but remainder_model is not created
+        # -> remainder_size must be zero (no need to create remainder_model)
+        assert remainder_size == 0, "remainder_size is non zero but no remainder_model has been created"
+        # remainder_size is not None only if called by convert_batch(...), so we return BatchContainer
+        # for this code path, even though there is no remainder_model created.
+        return BatchContainer(hb_container)
+
+    return hb_container
 
 
 class _PyTorchBackendModel(torch.nn.Module, object):
@@ -314,6 +453,10 @@ class _PyTorchBackendModel(torch.nn.Module, object):
         self._output_names = _fix_var_naming(reversed(operators), output_names, "output")
         self._operator_map = torch.nn.ModuleDict(operator_map)
         self._operators = operators
+        self.max_string_length = None
+
+        if constants.MAX_STRING_LENGTH in extra_config:
+            self.max_string_length = extra_config[constants.MAX_STRING_LENGTH]
 
     def forward(self, *inputs):
         with torch.no_grad():
@@ -332,22 +475,28 @@ class _PyTorchBackendModel(torch.nn.Module, object):
                 inputs = tuple(splits)
             inputs = [*inputs]
             variable_map = {}
-            device = _get_device(self)
+            device = get_device(self)
 
             # Maps data inputs to the expected variables.
             for i, input_name in enumerate(self._input_names):
-                if type(inputs[i]) is list:
-                    inputs[i] = np.array(inputs[i])
-                if type(inputs[i]) is np.ndarray:
-                    inputs[i] = torch.from_numpy(inputs[i])
-                    if inputs[i].dtype == torch.float64:
-                        # We convert double precision arrays into single precision. Sklearn does the same.
-                        inputs[i] = inputs[i].float()
-                elif type(inputs[i]) is not torch.Tensor:
-                    raise RuntimeError("Inputer tensor {} of not supported type {}".format(input_name, type(inputs[i])))
+                input_ = inputs[i]
+                if type(input_) is list:
+                    input_ = np.array(input_)
+                if type(input_) is np.ndarray:
+                    # Convert string arrays into int32.
+                    if input_.dtype.kind in constants.SUPPORTED_STRING_TYPES:
+                        assert self.max_string_length is not None
+
+                        input_ = from_strings_to_ints(input_, self.max_string_length)
+                    input_ = torch.from_numpy(input_)
+                elif type(input_) is not torch.Tensor:
+                    raise RuntimeError("Inputer tensor {} of not supported type {}".format(input_name, type(input_)))
+                if input_.dtype == torch.float64:
+                    # We convert double precision arrays into single precision. Sklearn does the same.
+                    input_ = input_.float()
                 if device is not None and device.type != "cpu":
-                    inputs[i] = inputs[i].to(device)
-                variable_map[input_name] = inputs[i]
+                    input_ = input_.to(device)
+                variable_map[input_name] = input_
 
             # Evaluate all the operators in the topology by properly wiring inputs \ outputs
             for operator in self._operators:
@@ -364,4 +513,4 @@ class _PyTorchBackendModel(torch.nn.Module, object):
             if len(self._output_names) == 1:
                 return variable_map[self._output_names[0]]
             else:
-                return list(variable_map[output_name] for output_name in self._output_names)
+                return tuple(variable_map[output_name] for output_name in self._output_names)
