@@ -25,10 +25,10 @@ class Scaler(BaseOperator, torch.nn.Module):
         self.scale = scale
 
         if offset is not None:
-            self.offset = torch.nn.Parameter(torch.FloatTensor([offset]), requires_grad=False)
+            self.offset = torch.nn.Parameter(torch.from_numpy(offset), requires_grad=False)
 
         if scale is not None:
-            self.scale = torch.nn.Parameter(torch.FloatTensor([scale]), requires_grad=False)
+            self.scale = torch.nn.Parameter(torch.from_numpy(scale), requires_grad=False)
 
     def forward(self, x):
         if self.offset is not None:
@@ -37,4 +37,4 @@ class Scaler(BaseOperator, torch.nn.Module):
         if self.scale is not None:
             x = x * self.scale
 
-        return x
+        return x.float()
