@@ -10,9 +10,10 @@ All operators, backends, and configurations settings supported in Hummingbird ar
 **Supported Backends**
 PyTorch,
 TorchScript,
-ONNX
+ONNX,
+TVM
 
-**Supported Operators**
+**Supported Operators (scikit-learn)**
 BernoulliNB,
 Binarizer,
 DecisionTreeClassifier,
@@ -30,6 +31,7 @@ KernelPCA,
 KBinsDiscretizer,
 KNeighborsClassifier,
 KNeighborsRegressor,
+LabelEncoder,
 LinearRegression,
 LinearSVC,
 LogisticRegression,
@@ -57,14 +59,31 @@ TreeEnsembleRegressor,
 TruncatedSVD,
 VarianceThreshold,
 
+**Supported Operators (LGBM)**
 LGBMClassifier,
 LGBMRanker,
 LGBMRegressor,
 
-
+**Supported Operators (XGB)**
 XGBClassifier,
 XGBRanker,
-XGBRegressor
+XGBRegressor,
+
+**Supported Operators (ONNX-ML)**
+"ArrayFeatureExtractor",
+"Binarizer"
+"Cast",
+"Concat",
+"FeatureVectorizer"
+"LabelEncoder",
+"LinearClassifier",
+"LinearRegressor",
+"OneHotEncoder",
+"Normalizer",
+"Reshape",
+"Scaler",
+"TreeEnsembleClassifier",
+"TreeEnsembleRegressor",
 """
 from collections import defaultdict
 
@@ -75,6 +94,7 @@ from ._utils import (
     lightgbm_installed,
     xgboost_installed,
     onnx_runtime_installed,
+    tvm_installed,
     sparkml_installed,
 )
 
@@ -133,6 +153,7 @@ def _build_sklearn_operator_list():
         from sklearn.preprocessing import (
             Binarizer,
             KBinsDiscretizer,
+            LabelEncoder,
             MaxAbsScaler,
             MinMaxScaler,
             Normalizer,
@@ -189,6 +210,7 @@ def _build_sklearn_operator_list():
             # Preprocessing
             Binarizer,
             KBinsDiscretizer,
+            LabelEncoder,
             MaxAbsScaler,
             MinMaxScaler,
             Normalizer,
@@ -273,6 +295,9 @@ def _build_onnxml_operator_list():
             "Reshape",
             # Preprocessing
             "ArrayFeatureExtractor",
+            "Binarizer",
+            "FeatureVectorizer",
+            "LabelEncoder",
             "OneHotEncoder",
             "Normalizer",
             "Scaler",
@@ -303,6 +328,11 @@ def _build_backend_map():
 
         backends[onnx.__name__] = onnx.__name__
 
+    if tvm_installed():
+        import tvm
+
+        backends[tvm.__name__] = tvm.__name__
+
     return backends
 
 
@@ -413,6 +443,11 @@ ONNX_OUTPUT_MODEL_NAME = "onnx_model_name"
 ONNX_TARGET_OPSET = "onnx_target_opset"
 """For ONNX models we can set the target opset to use. 11 by default."""
 
+TVM_MAX_FUSE_DEPTH = "tvm_max_fuse_depth"
+"""For TVM we can fix the number of operations that will be fused.
+If not set, compilation may take forever (https://github.com/microsoft/hummingbird/issues/232).
+By default Hummingbird uses a max_fuse_depth of 50, but this can be override using this parameter."""
+
 INPUT_NAMES = "input_names"
 """Set the names of the inputs. Assume that the numbers of inputs_names is equal to the number of inputs."""
 
@@ -428,3 +463,9 @@ Inter-op threads are by default set to 1 in Hummingbird. Check `tests.test_extra
 
 BATCH_SIZE = "batch_size"
 """Select whether to partition the input dataset at inference time in N batch_size partitions."""
+
+REMAINDER_SIZE = "remainder_size"
+"""Determines the number of rows that an auxiliary remainder model can accept."""
+
+MAX_STRING_LENGTH = "max_string_length"
+"""Maximum expected length for string features. By deafult this value is set using the training information."""
